:
#
# @(#)configure.sh	1.55 93/10/30
#
# Script to set up the Xmcd device-dependent configuration files.
#
#    xmcd - Motif(tm) CD Audio Player
#
#    Copyright (C) 1993  Ti Kan
#    E-mail: ti@amb.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

PATH=/bin:/usr/bin:/sbin:/usr/sbin:/etc:/usr/local/bin:/usr/ucb
export PATH

# Change the following directory to fit your local configuration
LIBDIR=/usr/lib/X11

VER=1.0


# Utility functions

getstr()
{
	$ECHO "$* \c"
	read ANS
	if [ -n "$ANS" ]
	then
		return 0
	else
		return 1
	fi
}

getyn()
{
	if [ -z "$YNDEF" ]
	then
		YNDEF=y
	fi

	while :
	do
		$ECHO "$*? [${YNDEF}] \c"
		read ANS
		if [ -n "$ANS" ]
		then
			case $ANS in
			[yY])
				RET=0
				break
				;;
			[nN])
				RET=1
				break
				;;
			*)
				$ECHO "Please answer y or n"
				;;
			esac
		else
			if [ $YNDEF = y ]
			then
				RET=0
			else
				RET=1
			fi
			break
		fi
	done

	YNDEF=
	return $RET
}


ask_config()
{
	$ECHO "\n  Since you have an unlisted CD-ROM drive, I will assume"
	$ECHO "  that it is SCSI-2 compliant.  If this is not true then"
	$ECHO "  xmcd will probably not work."
	$ECHO "\n  You will now be asked several technical questions about"
	$ECHO "  your CD-ROM drive.  If you don't know the answer, try"
	$ECHO "  accepting the default values, and if problems occur when"
	$ECHO "  using xmcd, reconfigure the settings by running this"
	$ECHO "  script again, or editing the $CONFIG file."
	$ECHO "\n  If you get an unlisted drive working with xmcd in this"
	$ECHO "  manner, the author of xmcd would like to hear from you"
	$ECHO "  and incorporate the settings into the next xmcd release."
	$ECHO "  Please send e-mail to \"ti@amb.org\"."

	VENDOR=0
	VOLBASE=0

	while :
	do
		$ECHO "\n  Does your CD-ROM drive on $XMCD_DEV support the following:\n"

		YNDEF=y
		if getyn "  - The Play_Audio_MSF SCSI command"
		then
			PLAYMSF=True
		else
			PLAYMSF=False
		fi

		YNDEF=n
		if getyn "  - The Play_Audio(12) SCSI command"
		then
			PLAY12=True
		else
			PLAY12=False
		fi

		YNDEF=y
		if getyn "  - The Play_Audio(10) SCSI command"
		then
			PLAY10=True
		else
			PLAY10=False
		fi

		YNDEF=y
		if getyn "  - The Play_Audio_Track/Index command"
		then
			PLAYTI=True
		else
			PLAYTI=False
		fi

		YNDEF=n
		if getyn "  - Caddy load via the Start_Stop_Unit SCSI command"
		then
			LOAD=True
		else
			LOAD=False
		fi

		YNDEF=y
		if getyn "  - Caddy eject via the Start_Stop_Unit SCSI command"
		then
			EJECT=True
		else
			EJECT=False
		fi

		YNDEF=y
		if getyn "  - Audio volume control via the Mode_Select SCSI command"
		then
			VOLCTL=True
		else
			VOLCTL=False
		fi

		YNDEF=n
		if getyn "  - Disable block descriptor in the Mode_Select SCSI command"
		then
			VOLDBD=True
		else
			VOLDBD=False
		fi

		YNDEF=y
		if getyn "  - The Pause/Resume SCSI command"
		then
			PAUSE=True
		else
			PAUSE=False
		fi

		YNDEF=y
		if getyn "  - The Prevent/Allow_Medium_Removal SCSI command"
		then
			CADDYLOCK=True
		else
			CADDYLOCK=False
		fi

		$ECHO "\n  This is the configuration for ${XMCD_DEV}:\n"
		$ECHO "  logicalDriverNumber:  $DRVNO"
		$ECHO "  playAudio12Support:   $PLAY12"
		$ECHO "  playAudioMSFSupport:  $PLAYMSF"
		$ECHO "  playAudio10Support:   $PLAY10"
		$ECHO "  playAudioTISupport:   $PLAYTI"
		$ECHO "  loadSupport:          $LOAD"
		$ECHO "  ejectSupport:         $EJECT"
		$ECHO "  volumeControlSupport: $VOLCTL"
		$ECHO "  volumeControlSetDBD:  $VOLDBD"
		$ECHO "  pauseResumeSupport:   $PAUSE"
		$ECHO "  caddyLockSupport:     $CADDYLOCK"

		YNDEF=y
		if getyn "\n  Is this acceptable"
		then
			break
		fi

		$ECHO "  Try again..."
	done
}


config_drive()
{
	MAXCHOICE=17

	$ECHO "\n  CD-ROM drive ($XMCD_DEV) selection:\n"
	$ECHO "        -: SCSI-1 mode"
	$ECHO "        *: SCSI-2 mode\n"
	$ECHO "     1. *  Apple   CD-  300"
	$ECHO "     2. *  Compaq  CDU- 561"
	$ECHO "     3. *  Compaq  CR-  503BCQ"
	$ECHO "     4. -  Hitachi CDR- 1650S, 1750S, 3650, 3750"
	$ECHO "     5. *  Hitachi CDR- 3750"
	$ECHO "     6. -  NEC     CDR- 72, 74, 77, 80, 82, 84, 74-1, 84-1"
	$ECHO "     7. *  NEC     CDR- 38"
	$ECHO "     8. *  NEC     CDR- 74-1, 84-1"
	$ECHO "     9. -  Pioneer DRM- 600, 600A, 610, 604X"
	$ECHO "    10. *  Pioneer DRM- 604X"
	$ECHO "    11. *  Sony    CDU- 541, 561, 6211, 7211, 8003, 8012"
	$ECHO "    12. *  Sun     -    CD-ROM"
	$ECHO "    13. *  Texel   DM-  3024, 5024"
	$ECHO "    14. *  Texel   DM-  3028, 5028"
	$ECHO "    15. -  Toshiba XM-  3101, 3201"
	$ECHO "    16. *  Toshiba XM-  3301, 3401, 8100"
	$ECHO "    17. *  Other (SCSI-2)"

	while :
	do
		$ECHO "\n  Enter choice: \c"
		read ANS
		if [ -z "$ANS" -o "$ANS" -lt 1 -o "$ANS" -gt $MAXCHOICE ]
		then
			$ECHO "  Please answer 1 to ${MAXCHOICE}."
		else
			case $ANS in
			1)
				# Apple CD-300
				VENDOR=0
				PLAYMSF=True
				PLAY12=False
				PLAY10=True
				PLAYTI=False
				LOAD=False
				EJECT=True
				VOLCTL=True
				VOLDBD=False
				VOLBASE=128
				PAUSE=True
				CADDYLOCK=True
				;;
			2)
				# Compaq CDU-561
				VENDOR=0
				PLAYMSF=True
				PLAY12=False
				PLAY10=True
				PLAYTI=False
				LOAD=False
				EJECT=True
				VOLCTL=True
				VOLDBD=False
				VOLBASE=128
				PAUSE=True
				CADDYLOCK=True
				;;
			3)
				# Compaq CR-503BCQ
				VENDOR=0
				PLAYMSF=True
				PLAY12=True
				PLAY10=True
				PLAYTI=False
				LOAD=False
				EJECT=True
				VOLCTL=True
				VOLDBD=False
				VOLBASE=0
				PAUSE=True
				CADDYLOCK=True
				;;
			4)
				# Hitachi VU
				VENDOR=4
				PLAYMSF=False
				PLAY12=False
				PLAY10=False
				PLAYTI=False
				LOAD=False
				EJECT=True
				VOLCTL=False
				VOLDBD=False
				VOLBASE=0
				PAUSE=False
				CADDYLOCK=True
				;;
			5)
				# Hitachi SCSI-2
				VENDOR=0
				PLAYMSF=True
				PLAY12=True
				PLAY10=True
				PLAYTI=True
				LOAD=False
				EJECT=True
				VOLCTL=False
				VOLDBD=False
				VOLBASE=0
				PAUSE=True
				CADDYLOCK=True
				;;
			6)
				# NEC VU
				VENDOR=2
				PLAYMSF=False
				PLAY12=False
				PLAY10=False
				PLAYTI=False
				LOAD=False
				EJECT=True
				VOLCTL=False
				VOLDBD=False
				VOLBASE=0
				PAUSE=False
				CADDYLOCK=True
				;;
			7)
				# NEC SCSI-2 (CDR-38)
				VENDOR=0
				PLAYMSF=True
				PLAY12=True
				PLAY10=True
				PLAYTI=False
				LOAD=False
				EJECT=False
				VOLCTL=True
				VOLDBD=True
				VOLBASE=0
				PAUSE=True
				CADDYLOCK=False
				;;
			8)
				# NEC SCSI-2
				VENDOR=0
				PLAYMSF=True
				PLAY12=True
				PLAY10=True
				PLAYTI=False
				LOAD=True
				EJECT=True
				VOLCTL=True
				VOLDBD=True
				PAUSE=True
				CADDYLOCK=True
				;;
			9)
				# Pioneer VU
				VENDOR=3
				PLAYMSF=False
				PLAY12=False
				PLAY10=False
				PLAYTI=False
				LOAD=False
				EJECT=True
				VOLCTL=False
				VOLDBD=False
				VOLBASE=0
				PAUSE=False
				CADDYLOCK=True
				;;
			10)
				# Pioneer SCSI-2
				VENDOR=0
				PLAYMSF=True
				PLAY12=True
				PLAY10=True
				PLAYTI=True
				LOAD=False
				EJECT=True
				VOLCTL=True
				VOLDBD=True
				VOLBASE=0
				PAUSE=True
				CADDYLOCK=True
				;;
			11)
				# Sony SCSI-2
				VENDOR=0
				PLAYMSF=True
				PLAY12=False
				PLAY10=True
				PLAYTI=False
				LOAD=False
				EJECT=True
				VOLCTL=True
				VOLDBD=False
				VOLBASE=128
				PAUSE=True
				CADDYLOCK=True
				;;
			12)
				# Sun CD-ROM
				VENDOR=0
				PLAYMSF=True
				PLAY12=False
				PLAY10=True
				PLAYTI=False
				LOAD=False
				EJECT=True
				VOLCTL=True
				VOLDBD=False
				VOLBASE=128
				PAUSE=True
				CADDYLOCK=True
				;;
			13)
				# Texel DM-x024
				VENDOR=0
				PLAYMSF=True
				PLAY12=False
				PLAY10=True
				PLAYTI=False
				LOAD=False
				EJECT=False
				VOLCTL=False
				VOLDBD=False
				VOLBASE=0
				PAUSE=True
				CADDYLOCK=False
				;;
			14)
				# Texel DM-x028
				VENDOR=0
				PLAYMSF=True
				PLAY12=False
				PLAY10=True
				PLAYTI=False
				LOAD=False
				EJECT=True
				VOLCTL=False
				VOLDBD=False
				VOLBASE=0
				PAUSE=True
				CADDYLOCK=True
				;;
			15)
				# Toshiba VU
				VENDOR=1
				PLAYMSF=False
				PLAY12=False
				PLAY10=False
				PLAYTI=False
				LOAD=False
				EJECT=True
				VOLCTL=False
				VOLDBD=False
				VOLBASE=0
				PAUSE=False
				CADDYLOCK=True
				;;
			16)
				# Toshiba SCSI-2
				VENDOR=0
				PLAYMSF=True
				PLAY12=True
				PLAY10=True
				PLAYTI=True
				LOAD=False
				EJECT=True
				VOLCTL=True
				VOLDBD=True
				VOLBASE=0
				PAUSE=True
				CADDYLOCK=True
				;;
			17)
				ask_config
				;;
			*)
				$ECHO "  Please answer 1 to ${MAXCHOICE}."
				;;
			esac

			break
		fi
	done
}



# Main starts here

# Use Sysv echo if possible
if [ -x /usr/5bin/echo ]
then
	ECHO=/usr/5bin/echo
else
	ECHO=echo
fi

# Use nawk if possible
if ($ECHO "x" | nawk '{ print $0 }' -) >/dev/null 2>&1
then
	AWK=nawk
else
	AWK=awk
fi

$ECHO "\nXmcd version $VER Device Configuration Program"

# Sanity check

(id | fgrep 'uid=0(root)') >/dev/null 2>&1
if [ $? != 0 ]
then
	$ECHO "Error: You must be the superuser to configure xmcd."
	exit 1
fi

if [ ! -r $LIBDIR/xmcd/config/dev.config ]
then
	$ECHO "Cannot find $LIBDIR/xmcd/config/dev.config !"
	$ECHO "aborting..."
	exit 2
fi

# Configure device-dependent parameters

OS_VER=`uname -r 2>/dev/null`
DEVBASE=/dev/rcdrom
DEVSUFFIX=
FIRST=0
LINKLIBS=no

if [ -x /bin/i386 -o -x /sbin/i386 ] && i386
then
	case $OS_VER in
	3.2)
		DEVBASE=/dev/rcd
		DEVSUFFIX=
		FIRST=0
		;;
	4.0)
		DEVBASE=/dev/rcdrom/cd
		DEVSUFFIX=
		FIRST=0
		LINKLIBS=yes
		;;
	4.[12])
		DEVBASE=/dev/rcdrom/cdrom
		DEVSUFFIX=
		FIRST=1
		LINKLIBS=yes
		;;
	4*MP)
		DEVBASE=/dev/rcdrom/cdrom
		DEVSUFFIX=
		FIRST=1
		LINKLIBS=yes
		;;
	*)
		OS_VER=unknown
		;;
	esac
elif [ -x /bin/m88k ] && m88k
then
	case $OS_VER in
	4.0)
		DEVBASE=/dev/rdsk/m187_c0d
		DEVSUFFIX=s7
		FIRST=3
		LINKLIBS=yes
		;;
	*)
		OS_VER=unknown
		;;
	esac
elif [ -x /bin/hp-pa ] && hp-pa
then
	case $OS_VER in
	A.09*)
		DEVBASE=/dev/rdsk/c201d
		DEVSUFFIX=s0
		FIRST=4
		;;
	*)
		OS_VER=unknown
		;;
	esac
elif [ -x /bin/sun ] && sun
then
	case $OS_VER in
	4.*)
		case `arch -k` in
		sun4[cm])
			DEVBASE=/dev/rsr
			DEVSUFFIX=
			FIRST=0
			;;
		*)
			OS_VER=unknown
			;;
		esac
		;;
	5.*)
		DEVBASE=/dev/rdsk/c0t
		DEVSUFFIX=d0s0
		FIRST=6
		LINKLIBS=yes
		;;
	*)
		OS_VER=unknown
		;;
	esac
elif [ -x /bin/ftx ] && ftx
then
	case $OS_VER in
	4.0)
		DEVBASE=/dev/rcdrom/c2a1d
		DEVSUFFIX=s0
		FIRST=0
		LINKLIBS=yes
		;;
	*)
		OS_VER=unknown
		;;
	esac
else
	OS_VER=unknown
fi

NOT_SUPPORTED="Error: You are not running an operating system that's\n\
currently supported by xmcd."

if [ "$OS_VER" = unknown ]
then
	$ECHO "$NOT_SUPPORTED"
	YNDEF=n
	getyn "Would you like to proceed anyway?"
	if [ $? -ne 0 ]
	then
		$ECHO "Configuration aborted."
		exit 3
	fi
fi

DRVNO=0
DEVNO=$FIRST
SEDLINE=
while :
do
	$ECHO "\nConfiguring SCSI CD-ROM drive $DRVNO..."

	DEFAULT_DEV="${DEVBASE}${DEVNO}${DEVSUFFIX}"

	while :
	do
		if getstr "\n  Enter raw CD-ROM device: [$DEFAULT_DEV]"
		then
			XMCD_DEV=$ANS
		else
			XMCD_DEV=$DEFAULT_DEV
		fi

		if [ -c $XMCD_DEV ]
		then
			break
		else
			$ECHO "  $XMCD_DEV is an invalid device."
		fi
	done

	if [ $DRVNO -eq 0 ]
	then
		$AWK '{
			if (match($0, "^XMcd.device:") > 0)
				printf("XMcd.device:\t\t\t%s\n", device)
			else if (match($0, "^XMcd.libdir:") > 0)
				printf("XMcd.libdir:\t\t\t%s/xmcd\n", libdir)
			else
				print $0
		}' device=$XMCD_DEV libdir=$LIBDIR \
			$LIBDIR/app-defaults/XMcd > /tmp/xmcd.$$

		cp /tmp/xmcd.$$ $LIBDIR/app-defaults/XMcd
		rm -f /tmp/xmcd.$$

		if [ $LIBDIR != "/usr/lib/X11" -a \
		     -d /usr/lib/X11/app-defaults ]
		then
			rm -f /usr/lib/X11/app-defaults/._trash_
			>$LIBDIR/app-defaults/._trash_

			if [ ! -f /usr/lib/X11/app-defaults/._trash_ ]
			then
				rm -f /usr/lib/X11/app-defaults/XMcd
				ln $LIBDIR/app-defaults/XMcd \
					/usr/lib/X11/app-defaults/XMcd
			fi

			rm -f $LIBDIR/app-defaults/._trash_
		fi
	fi

	CONFIG=$LIBDIR/xmcd/config/`basename $XMCD_DEV`

	config_drive

	DRVNOTICE="\n\n  NOTE: This drive does not support these features:"

	if [ $PLAYTI = False ]
	then
		$ECHO "$DRVNOTICE"
		DRVNOTICE="\c"
		$ECHO "  - The Previous Index and Next Index buttons."
	fi

	if [ $CADDYLOCK = False ]
	then
		$ECHO "$DRVNOTICE"
		DRVNOTICE="\c"
		$ECHO "  - The caddy lock."
	fi

	if [ $LOAD = False ]
	then
		$ECHO "$DRVNOTICE"
		DRVNOTICE="\c"
		$ECHO "  - Software-controlled caddy load."
	fi

	if [ $VOLCTL = False ]
	then
		$ECHO "$DRVNOTICE"
		DRVNOTICE="\c"
		$ECHO "  - The volume control slider."
	fi

	$ECHO "\n  Creating the $CONFIG file..."

	sed \
	   -e "s/^logicalDriveNumber:.*/logicalDriveNumber:	$DRVNO/" \
	   -e "s/^driveVendorCode:.*/driveVendorCode:	$VENDOR/" \
	   -e "s/^playAudio12Support:.*/playAudio12Support:	$PLAY12/" \
	   -e "s/^playAudioMSFSupport:.*/playAudioMSFSupport:	$PLAYMSF/" \
	   -e "s/^playAudio10Support:.*/playAudio10Support:	$PLAY10/" \
	   -e "s/^playAudioTISupport:.*/playAudioTISupport:	$PLAYTI/" \
	   -e "s/^loadSupport:.*/loadSupport:		$LOAD/" \
	   -e "s/^ejectSupport:.*/ejectSupport:		$EJECT/" \
	   -e "s/^volumeControlSupport:.*/volumeControlSupport:	$VOLCTL/" \
	   -e "s/^volumeControlSetDBD:.*/volumeControlSetDBD:	$VOLDBD/" \
	   -e "s/^scsiAudioVolumeBase:.*/scsiAudioVolumeBase:	$VOLBASE/" \
	   -e "s/^pauseResumeSupport:.*/pauseResumeSupport:	$PAUSE/" \
	   -e "s/^caddyLockSupport:.*/caddyLockSupport:	$CADDYLOCK/" \
	   < $LIBDIR/xmcd/config/dev.config > $CONFIG
	chown bin $CONFIG
	chgrp bin $CONFIG
	chmod 644 $CONFIG

	YNDEF=n
	if getyn "\n  Do you have more SCSI CD-ROM drives on your system"
	then
		DRVNO=`expr $DRVNO + 1`
		DEVNO=`expr $DEVNO + 1`
	else
		break
	fi
done


if [ $LINKLIBS = yes -a -d $LIBDIR -a $LIBDIR != /usr/lib/X11 ]
then
	#
	# For security, SVR4 setuid programs search only /usr/lib and
	# /usr/ccs/lib for shared libraries, but if the X shared
	# libraries are not in these directories then symlinks
	# are needed to make xmcd run.
	#
	cd $LIBDIR
	for i in libX*.so*
	do
		if [ "$i" = 'libX*.so*' ]
		then
			break
		fi

		if [ ! -f /usr/lib/$i ]
		then
			ln -s $LIBDIR/$i /usr/lib/$i
			$ECHO "\nLinked $LIBDIR/$i -> /usr/lib/$i"
		fi
	done
fi

$ECHO "\n  Xmcd set-up is now complete.\n"
$ECHO "  Please read the README file supplied with the xmcd"
$ECHO "  distribution for hardware configuration information about"
$ECHO "  specific CD-ROM drives.\n"
exit 0
