:
#
# @(#)install.sh	1.28 93/11/05
#
# Script to install xmcd software binary and support files on
# the target system.
#
#    xmcd - Motif(tm) CD Audio Player
#
#    Copyright (C) 1993  Ti Kan
#    E-mail: ti@amb.org
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#

PATH=/bin:/usr/bin:/sbin:/usr/sbin:/etc:/usr/local/bin
export PATH

CATEGORIES="rock jazz newage classical soundtrack misc data"
XMCD_VER="1.0"


#
# Utility functions
#

getstr()
{
	$ECHO "$* \c"
	read ANS
	if [ -n "$ANS" ]
	then
		return 0
	else
		return 1
	fi
}

getyn()
{
	if [ -z "$YNDEF" ]
	then
		YNDEF=y
	fi

	while :
	do
		$ECHO "$*? [${YNDEF}] \c"
		read ANS
		if [ -n "$ANS" ]
		then
			case $ANS in
			[yY])
				RET=0
				break
				;;
			[nN])
				RET=1
				break
				;;
			*)
				$ECHO "Please answer y or n"
				;;
			esac
		else
			if [ $YNDEF = y ]
			then
				RET=0
			else
				RET=1
			fi
			break
		fi
	done

	YNDEF=
	return $RET
}


#
# Main execution starts here
#

# Use SysV echo if possible
if [ -x /usr/5bin/echo ]
then
	ECHO=/usr/5bin/echo
else
	ECHO=echo
fi

# Use nawk if possible
if ($ECHO "x" | nawk '{ print $0 }' -) >/dev/null 2>&1
then
	AWK=nawk
else
	AWK=awk
fi

NOREMOVE=0
if [ $# -eq 1 -a "$1" = "-n" ]
then
	NOREMOVE=1
fi

# Sanity check

(id | fgrep 'uid=0(root)') >/dev/null 2>&1
if [ $? != 0 ]
then
	$ECHO "Error: You must be the superuser to install xmcd."
	exit 1
fi

if [ ! -f xmcd -o ! -f XMcd.ad -o ! -f dev.config -o ! -f configure.sh ]
then
	$ECHO "Error: missing files."
	exit 3
fi


if (uname -X | fgrep "Release = 3.2") >/dev/null 2>&1
then
	SCO=1
else
	SCO=0
fi


$ECHO "Installing \"xmcd\" Motif CD Player version $XMCD_VER by Ti Kan"


# Determine BINDIR

if [ -z "$BINDIR" ]
then
	BINDIR=/usr/bin/X11

	if [ -d /usr/X/bin ]
	then
		BINDIR=/usr/X/bin
	elif [ -d /usr/X386/bin ]
	then
		BINDIR=/usr/X386/bin
	elif [ -d /usr/local/bin/X11 ]
	then
		BINDIR=/usr/local/bin/X11
	elif [ -d /usr/local/bin ]
	then
		BINDIR=/usr/local/bin
	elif [ -d /usr/lbin ]
	then
		BINDIR=/usr/lbin
	fi
else
	BINDIR=`echo $BINDIR | sed 's/\/\//\//g'`
fi

while :
do
	if getstr "\nEnter X binary directory [${BINDIR}]:"
	then
		if [ -d "$ANS" ]
		then
			BINDIR=$ANS
			break
		else
			$ECHO "Error: $ANS does not exist."
		fi
	else
		break
	fi
done


# Determine LIBDIR

if [ -z "$LIBDIR" ]
then
	LIBDIR=/usr/lib/X11

	if [ -d /usr/X/lib ]
	then
		LIBDIR=/usr/X/lib
	elif [ -d /usr/X386/lib/X11 ]
	then
		LIBDIR=/usr/X386/lib/X11
	elif [ -d /usr/local/lib/X11 ]
	then
		LIBDIR=/usr/local/lib/X11
	fi
else
	LIBDIR=`echo $LIBDIR | sed 's/\/\//\//g'`
fi

while :
do
	if getstr "\nEnter X library directory [${LIBDIR}]:"
	then
		if [ -d "$ANS" ]
		then
			LIBDIR=$ANS
			break
		else
			$ECHO "Error: $ANS does not exist."
		fi
	else
		break
	fi
done


# Determine MANFILE

if [ -z "$MANFILE" ]
then
	MANFILE=/usr/man/man1/xmcd.1

	if [ -d /usr/man/man.LOCAL ]
	then
		MANFILE=/usr/man/man.LOCAL/xmcd.LOCAL
	elif [ -d /usr/local/man/man1 ]
	then
		MANFILE=/usr/local/man/man1/xmcd.1
	fi
else
	MANFILE=`echo $MANFILE | sed 's/\/\//\//g'`
fi

if getstr "\nEnter on-line manual file path [${MANFILE}]:"
then
	MANFILE=$ANS
fi

MANDIR=`dirname $MANFILE`
if [ ! -d $MANDIR ]
then
	YNDEF=y
	if getyn "Directory $MANDIR does not exist.  Create it"
	then
		(umask 022; mkdir -p $MANDIR)
	else
		$ECHO "The on-line xmcd manual will not be installed."
		MANFILE=
		MANDIR=
	fi
fi


$ECHO "\nMaking directories..."

# Make all necessary directories

$ECHO "\t$LIBDIR/xmcd"
if [ ! -d $LIBDIR/xmcd ]
then
	mkdir $LIBDIR/xmcd
fi
chown bin $LIBDIR/xmcd
chgrp bin $LIBDIR/xmcd
chmod 755 $LIBDIR/xmcd

$ECHO "\t$LIBDIR/xmcd/config"
if [ ! -d $LIBDIR/xmcd/config ]
then
	mkdir $LIBDIR/xmcd/config
fi
chown bin $LIBDIR/xmcd/config
chgrp bin $LIBDIR/xmcd/config
chmod 755 $LIBDIR/xmcd/config

$ECHO "\t$LIBDIR/xmcd/help"
if [ ! -d $LIBDIR/xmcd/help ]
then
	mkdir $LIBDIR/xmcd/help
fi
chown bin $LIBDIR/xmcd/help
chgrp bin $LIBDIR/xmcd/help
chmod 755 $LIBDIR/xmcd/help

$ECHO "\t$LIBDIR/xmcd/cddb"
if [ ! -d $LIBDIR/xmcd/cddb ]
then
	mkdir $LIBDIR/xmcd/cddb
fi
chown bin $LIBDIR/xmcd/cddb
chgrp bin $LIBDIR/xmcd/cddb
chmod 755 $LIBDIR/xmcd/cddb

for i in $CATEGORIES
do
	$ECHO "\t$LIBDIR/xmcd/cddb/$i"
	if [ ! -d $LIBDIR/xmcd/cddb/$i ]
	then
		mkdir $LIBDIR/xmcd/cddb/$i
	fi
	chown bin $LIBDIR/xmcd/cddb/$i
	chgrp bin $LIBDIR/xmcd/cddb/$i
	chmod 777 $LIBDIR/xmcd/cddb/$i
done

# Install files
$ECHO "\nInstalling xmcd files..."

$ECHO "\t$BINDIR/xmcd"
cp xmcd $BINDIR/xmcd
chown root $BINDIR/xmcd
chgrp bin $BINDIR/xmcd
chmod 4711 $BINDIR/xmcd

$ECHO "\t$LIBDIR/app-defaults/XMcd"
cp XMcd.ad $LIBDIR/app-defaults/XMcd
chown bin $LIBDIR/app-defaults/XMcd
chgrp bin $LIBDIR/app-defaults/XMcd
chmod 444 $LIBDIR/app-defaults/XMcd
if [ $LIBDIR != "/usr/lib/X11" -a -d /usr/lib/X11/app-defaults ]
then
	# Test LIBDIR
	rm -f /usr/lib/X11/app-defaults/._junk_

	>$LIBDIR/app-defaults/._junk_

	if [ ! -f /usr/lib/X11/app-defaults/._junk_ ]
	then
		rm -f /usr/lib/X11/app-defaults/XMcd
		ln $LIBDIR/app-defaults/XMcd /usr/lib/X11/app-defaults/XMcd
	fi

	rm -f $LIBDIR/app-defaults/._junk_
fi

$ECHO "\t$LIBDIR/xmcd/help/*"
cp helpfiles/*Btn helpfiles/*Ind helpfiles/*Txt helpfiles/*Scale \
	helpfiles/*List helpfiles/*Box $LIBDIR/xmcd/help
chown bin $LIBDIR/xmcd/help/*
chgrp bin $LIBDIR/xmcd/help/*
chmod 444 $LIBDIR/xmcd/help/*

$ECHO "\t$LIBDIR/xmcd/cddb/misc/4102560a"
cp demo.cddb $LIBDIR/xmcd/cddb/misc/4102560a
chown bin $LIBDIR/xmcd/cddb/misc/4102560a
chgrp bin $LIBDIR/xmcd/cddb/misc/4102560a
chmod 444 $LIBDIR/xmcd/cddb/misc/4102560a

$ECHO "\t$LIBDIR/xmcd/config/dev.config"
cp dev.config $LIBDIR/xmcd/config/dev.config
chown bin $LIBDIR/xmcd/config/dev.config
chgrp bin $LIBDIR/xmcd/config/dev.config
chmod 444 $LIBDIR/xmcd/config/dev.config

$ECHO "\t$LIBDIR/xmcd/config/configure.sh"
$AWK '
	/^LIBDIR=/	{ printf("LIBDIR=%s\n", libdir) }
	!/^LIBDIR=/	{ print $0 }
' libdir=$LIBDIR configure.sh >$LIBDIR/xmcd/config/configure.sh
chown bin $LIBDIR/xmcd/config/configure.sh
chgrp bin $LIBDIR/xmcd/config/configure.sh
chmod 755 $LIBDIR/xmcd/config/configure.sh

if [ ! -f $LIBDIR/XKeysymDB ]
then
	$ECHO "\t$LIBDIR/XKeysymDB"
	cp XKeysymDB $LIBDIR/XKeysymDB
	chown bin $LIBDIR/XKeysymDB
	chgrp bin $LIBDIR/XKeysymDB
	chmod 644 $LIBDIR/XKeysymDB
fi

if [ "$SCO" = 1 ]
then
	$ECHO "\t/usr/lib/mkdev/xmcd"
	rm -f /usr/lib/mkdev/xmcd
	ln $LIBDIR/xmcd/config/configure.sh /usr/lib/mkdev/xmcd
fi

if [ -n "$MANFILE" ]
then
	$ECHO "\t$MANFILE"
	cp xmcd.man $MANFILE
	chown bin $MANFILE
	chgrp bin $MANFILE
	chmod 644 $MANFILE
fi

# Run device-dependent config script
sh $LIBDIR/xmcd/config/configure.sh

if [ $NOREMOVE = 0 ]
then
	rm -rf xmcd XMcd.ad demo.cddb dev.config helpfiles install.sh \
		configure.sh XKeysymDB xmcd.man xmcd.tar
fi
$ECHO "\nInstallation of xmcd is now complete."

exit 0
