/*
 *   xmcd - Motif(tm) CD Audio Player
 *
 *   Copyright (C) 1993  Ti Kan
 *   E-mail: ti@amb.org
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *   The name "Hitachi" is a trademark of Hitachi Corporation, and is
 *   used here for identification purposes.  This software and its
 *   author are not affiliated in any way with Hitachi.
 *
 */
#ifndef LINT
static char *_lib_hita_c_ident_ = "@(#)lib_hita.c	1.36 93/09/28";
#endif

#include <Xm/Xm.h>
#include "xmcd.h"
#include "util.h"
#include "cdfunc.h"
#include "lib_scsipt.h"

#ifdef HITACHI

#ifndef LINT
bool_t			lib_hita = TRUE;
#endif


STATIC bool_t		hita_paused = FALSE,		/* Currently paused */
			hita_playing = FALSE,		/* Currently playing */
			hita_audio_muted = FALSE;
							/* Audio is muted */
STATIC word32_t		hita_pause_addr = 0;		/* Pause addr */
STATIC haudio_arg_t	hita_sav_end;			/* Save addr */


/*
 * Internal functions
 */

/*
 * hita_do_pause
 *	Send a vendor-unique Pause command to the drive
 *
 * Args:
 *	ret_addr - Pointer to a buffer where the paused address will be
 *		   written to.  If NULL, no pause address info will
 *		   returned.
 *
 * Return:
 *	TRUE - success
 *	FALSE - failure
 */
STATIC bool_t
hita_do_pause(hmsf_t *ret_addr)
{
	hmsf_t	pause_addr;
	bool_t	ret;

	if ((ret = pthru_send(OP_VH_PAUSE, 0,
			      (byte_t *) AD_VH_PAUSE(&pause_addr),
			      SZ_VH_PAUSE, 0, 0, 0, 0, READ_OP)) == TRUE) {
		if (ret_addr != NULL)
			*ret_addr = pause_addr;	/* structure copy */
	}
	return(ret);
}


/*
 * Public functions
 */

/*
 * hita_playaudio
 *	Play audio function: send vendor-unique play audio command
 *	to the drive.
 *
 * Args:
 *	addr_fmt - Flags indicating which address formats are passed in
 *	If ADDR_BLK, then:
 *	    start_addr - The logical block starting address
 *	    end_addr - The logical block ending address
 *	If ADD_MSF, then:
 *	    start_msf - Pointer to the starting MSF address structure
 *	    end_msf - Pointer to the ending MSF address structure
 *	If ADDR_TRKIDX, then:
 *	    trk - The starting track number
 *	    idx - The starting index number
 *	If ADDR_OPTEND, then the ending address, if specified, can be
 *	ignored if possible.
 *
 * Return:
 *	TRUE - success
 *	FALSE - failure
 */
/*ARGSUSED*/
bool_t
hita_playaudio(
	byte_t		addr_fmt,
	word32_t	start_addr,
	word32_t	end_addr,
	msf_t		*start_msf,
	msf_t		*end_msf,
	byte_t		trk,
	byte_t		idx
)
{
	bool_t		ret = FALSE;
	msf_t		istart_msf,
			iend_msf;
	word32_t	addr1 = 0,
			addr2 = 0;
	haudio_arg_t	*a1,
			*a2;
	curstat_t	*s = curstat_addr();

	a1 = (haudio_arg_t *) &addr1;
	a2 = (haudio_arg_t *) &addr2;

	if (!ret && (addr_fmt & ADDR_BLK) && !(addr_fmt & ADDR_MSF)) {
		/* Convert block address to MSF format */
		blktomsf(
			start_addr,
			&istart_msf.min, &istart_msf.sec, &istart_msf.frame,
			MSF_OFFSET(s)
		);

		blktomsf(
			end_addr,
			&iend_msf.min, &iend_msf.sec, &iend_msf.frame,
			MSF_OFFSET(s)
		);

		/* Let the ADDR_MSF code handle the request */
		start_msf = &istart_msf;
		end_msf = &iend_msf;
		addr_fmt |= ADDR_MSF;
		ret = FALSE;
	}

	if (!ret && (addr_fmt & ADDR_MSF)) {
		a1->addr_smin = (byte_t) start_msf->min;
		a1->addr_ssec = (byte_t) start_msf->sec;
		a1->addr_sframe = (byte_t) start_msf->frame;

		a2->addr_emin = hita_sav_end.addr_emin = (byte_t) end_msf->min;
		a2->addr_esec = hita_sav_end.addr_esec = (byte_t) end_msf->sec;
		a2->addr_eframe = hita_sav_end.addr_eframe = (byte_t)
			end_msf->frame;

		/* Send a pause command to cease any current audio playback,
		 * then send the actual play audio command.
		 */
		if ((ret = hita_do_pause(NULL)) == TRUE) {
			ret = pthru_send(
				OP_VH_AUDPLAY,
				addr1, NULL, 0, 0, addr2,
				(byte_t) (hita_audio_muted ? 0x7 : 0x1),
				0, READ_OP
			);
		}
	}

	if (ret) {
		hita_paused = FALSE;
		hita_playing = TRUE;
	}

	return(ret);
}


/*
 * hita_pause_resume
 *	Pause/resume function: send vendor-unique commands to implement
 *	the pause and resume capability.
 *
 * Args:
 *	resume - TRUE: resume, FALSE: pause
 *
 * Return:
 *	TRUE - success
 *	FALSE - failure
 */
bool_t
hita_pause_resume(bool_t resume)
{
	bool_t		ret = FALSE;
	word32_t	addr1 = 0,
			addr2 = 0;
	haudio_arg_t	*a1,
			*a2;
	hmsf_t		*a;


	a1 = (haudio_arg_t *) &addr1;
	a2 = (haudio_arg_t *) &addr2;
	a = (hmsf_t *) &hita_pause_addr;

        if (resume) {
		if (!hita_paused)
			return(TRUE);

		a1->addr_smin = a->min;
		a1->addr_ssec = a->sec;
		a1->addr_sframe = a->frame;
		a2->addr_emin = hita_sav_end.addr_emin;
		a2->addr_esec = hita_sav_end.addr_esec;
		a2->addr_eframe = hita_sav_end.addr_eframe;

		ret = pthru_send(
			OP_VH_AUDPLAY,
			addr1, NULL, 0, 0, addr2,
			(byte_t) (hita_audio_muted ? 0x7 : 0x1),
			0, READ_OP
		);
        }
        else {
		if (hita_paused)
			return(TRUE);

		ret = hita_do_pause(a);
	}

	if (ret) {
		hita_paused = !resume;
		hita_playing = !hita_paused;
	}

	return(ret);
}


/*
 * hita_start_stop
 *	Start/stop function: When playing audio, the Hitachi drive must
 *	first be paused before sending a Start/Stop Unit command to
 *	stop it.  This routine performs the pause.
 *
 * Args:
 *	start - TRUE: start unit, FALSE: stop unit
 *	loej - TRUE: load/eject caddy, FALSE: do not load/eject (not used)
 *
 * Return:
 *	TRUE - success
 *	FALSE - failure
 */
/*ARGSUSED*/
bool_t
hita_start_stop(bool_t start, bool_t loej)
{
	/* If audio playback is in progress, pause the playback.
	 * Then, return to caller (do_start_stop() in lib_scsipt.c)
	 * and issue a start/stop unit command from there.
	 */
	if (!start && hita_playing) {
		hita_playing = FALSE;

		return(hita_do_pause(NULL));
	}

	hita_paused = FALSE;
	return(TRUE);
}


/*
 * hita_get_playstatus
 *	Send vendor-unique command to obtain current audio playback
 *	status.
 *
 * Args:
 *	s - Pointer to the curstat_t structure
 *	audio_status - Address where a current status code (SCSI-2
 *		       style) is to be returned.
 *
 * Return:
 *	TRUE - success
 *	FALSE - failure
 */
bool_t
hita_get_playstatus(curstat_t *s, byte_t *audio_status)
{
	int		trkno,
			idxno;
	byte_t		buf[SZ_VH_RDSTAT];
	haudstat_t	*d;


	memset(buf, (byte_t) 0, sizeof(buf));

	if (!pthru_send(OP_VH_RDSTAT, 0, (byte_t *) AD_VH_RDSTAT(buf),
			SZ_VH_RDSTAT, 0, 0, 0, 0, READ_OP))
		return(FALSE);

	d = (haudstat_t *)(void *) buf;

	trkno = (word32_t) d->trkno;
	if (s->cur_trk != trkno) {
		s->cur_trk = trkno;
		dpy_track(s);
	}
	idxno = 1;	/* Fudge */
	if (s->cur_idx != idxno) {
		s->cur_idx = idxno;
		dpy_index(s);
	}

	s->cur_tot_min = (byte_t) d->abs_addr.min;
	s->cur_tot_sec = (byte_t) d->abs_addr.sec;
	s->cur_tot_frame = (byte_t) d->abs_addr.frame;
	s->cur_trk_min = (byte_t) d->rel_addr.min;
	s->cur_trk_sec = (byte_t) d->rel_addr.sec;
	s->cur_trk_frame = (byte_t) d->rel_addr.frame;
	msftoblk(
		s->cur_tot_min, s->cur_tot_sec, s->cur_tot_frame,
		&s->cur_tot_addr, MSF_OFFSET(s)
	);
	msftoblk(
		s->cur_trk_min, s->cur_trk_sec, s->cur_trk_frame,
		&s->cur_trk_addr, 0
	);

	/* Make up SCSI-2 style audio status */
	if (hita_paused)
		*audio_status = AUDIO_PAUSED;
	else if (d->playing)
		*audio_status = AUDIO_PLAYING;
	else {
		*audio_status = AUDIO_COMPLETED;
		hita_playing = FALSE;
	}

	return(TRUE);
}


/*
 * hita_get_toc
 *	Send vendor-unique command to obtain the disc table-of-contents
 *
 * Args:
 *	s - Pointer to the curstat_t structure, which contains the TOC
 *	    table to be updated.
 *
 * Return:
 *	TRUE - success
 *	FALSE - failure
 */
bool_t
hita_get_toc(curstat_t *s)
{
	int		i,
			j,
			xfer_len;
	byte_t		buf[sizeof(hdiscinfo_t)];
	hdiscinfo_t	*p;
	hmsf_t		*a;


	if (hita_playing)
		return(FALSE);	/* Drive is busy */

	memset(buf, (byte_t) 0, sizeof(buf));

	/* Read the TOC header first */
	if (!pthru_send(OP_VH_RDINFO, 0, (byte_t *) AD_VH_RDINFO(buf),
			SZ_VH_TOCHDR, SZ_VH_TOCHDR & 0xff,
			SZ_VH_TOCHDR >> 8, 0, 0, READ_OP))
		return(FALSE);

	p = (hdiscinfo_t *)(void *) buf;

	s->first_trk = (byte_t) p->first_trk;
	s->last_trk = (byte_t) p->last_trk;

	xfer_len = SZ_VH_TOCHDR +
		   ((int) (p->last_trk - p->first_trk + 2) * SZ_VH_TOCENT);

	if (xfer_len > SZ_VH_RDINFO)
		xfer_len = SZ_VH_RDINFO;

	/* Read the appropriate number of bytes of the entire TOC */
	if (!pthru_send(OP_VH_RDINFO, 0, (byte_t *) AD_VH_RDINFO(buf),
			xfer_len, (byte_t) (xfer_len & 0xff), xfer_len >> 8,
			0, 0, READ_OP))
		return(FALSE);

	/* Get the starting position of each track */
	for (i = 0, j = (int) s->first_trk; j <= (int) s->last_trk; i++, j++) {
		a = (hmsf_t *)(void *) &p->msfdata[(i+1) * 3];
		s->trkinfo[i].trkno = j;
		s->trkinfo[i].min = (byte_t) a->min;
		s->trkinfo[i].sec = (byte_t) a->sec;
		s->trkinfo[i].frame = (byte_t) a->frame;
		msftoblk(
			s->trkinfo[i].min,
			s->trkinfo[i].sec,
			s->trkinfo[i].frame,
			&s->trkinfo[i].addr,
			MSF_OFFSET(s)
		);
		s->trkinfo[i].type = (a->data == 0) ? TYP_AUDIO : TYP_DATA;
	}
	s->tot_trks = (byte_t) i;

	/* Get the lead-out track position */
	a = (hmsf_t *)(void *) &p->msfdata[0];
	s->trkinfo[i].trkno = LEAD_OUT_TRACK;
	s->tot_min = s->trkinfo[i].min = (byte_t) a->min;
	s->tot_sec = s->trkinfo[i].sec = (byte_t) a->sec;
	s->tot_frame = s->trkinfo[i].frame = (byte_t) a->frame;
	msftoblk(
		s->trkinfo[i].min,
		s->trkinfo[i].sec,
		s->trkinfo[i].frame,
		&s->trkinfo[i].addr,
		MSF_OFFSET(s)
	);
	s->tot_addr = s->trkinfo[i].addr;

	return(TRUE);
}


/*
 * hita_mute
 *	Send vendor-unique command to mute/unmute the audio
 *
 * Args:
 *	mute - TRUE: mute audio, FALSE: un-mute audio
 *
 * Return:
 *	TRUE - success
 *	FALSE - failure
 */
bool_t
hita_mute(bool_t mute)
{
	word32_t	addr1 = 0,
			addr2 = 0;
	haudio_arg_t	*a1,
			*a2;
	curstat_t	*s = curstat_addr();


	if (mute == hita_audio_muted)
		return(TRUE);

	a1 = (haudio_arg_t *) &addr1;
	a2 = (haudio_arg_t *) &addr2;

	if (hita_playing) {
		/* Pause the playback first */
        	if (!hita_do_pause(NULL))
			return(FALSE);

		a1->addr_smin = (byte_t) s->cur_tot_min;
		a1->addr_ssec = (byte_t) s->cur_tot_sec;
		a1->addr_sframe = (byte_t) s->cur_tot_frame;
		a2->addr_emin = hita_sav_end.addr_emin;
		a2->addr_esec = hita_sav_end.addr_esec;
		a2->addr_eframe = hita_sav_end.addr_eframe;

		if (!pthru_send(OP_VH_AUDPLAY, addr1, NULL, 0, 0, addr2,
				(byte_t) (mute ? 0x7 : 0x1), 0, READ_OP))
			return(FALSE);
	}

	hita_audio_muted = mute;

	return(TRUE);
}


/*
 * hita_eject
 *	Send vendor-unique command to eject the caddy
 *
 * Args:
 *	Nothing.
 *
 * Return:
 *	TRUE - success
 *	FALSE - failure
 */
bool_t
hita_eject(void)
{
	/* If audio playback is in progress, pause the playback first */
	if (hita_playing && !hita_do_pause(NULL))
		return(FALSE);

	hita_playing = hita_paused = FALSE;

	/* Eject the caddy */
	return(pthru_send(OP_VH_EJECT, 0, NULL, 0, 0x1, 0, 0, 0, READ_OP));
}


/*
 * hita_init
 *	Initialize the vendor-unique support module
 *
 * Args:
 *	Nothing.
 *
 * Return:
 *	Nothing.
 */
void
hita_init(void)
{
	/* Do nothing */
}


/*
 * hita_halt
 *	Shut down the vendor-unique support module
 *
 * Args:
 *	Nothing.
 *
 * Return:
 *	Nothing.
 */
void
hita_halt(void)
{
	/* Do nothing */
}


#else	/* !HITACHI */

#ifndef LINT
bool_t				lib_hita = FALSE;
#endif

#endif	/* HITACHI */

