/*
 *   xmcd - Motif(tm) CD Audio Player
 *
 *   Copyright (C) 1993  Ti Kan
 *   E-mail: ti@amb.org
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 */
#ifndef __LIB_SCSIPT_H__
#define __LIB_SCSIPT_H__

#ifndef LINT
static char *_lib_scsipt_h_ident_ = "@(#)lib_scsipt.h	1.37 93/09/28";
#endif

#ifndef CDLIB_MODULE
#define CDLIB_MODULE
#endif


/* Vendor unique operations code */
#define VENDOR_SCSI2		0	/* Use SCSI-2 audio commands */
#define VENDOR_TOSHIBA		1	/* Use Toshiba audio commands */
#define VENDOR_NEC		2	/* Use NEC audio commands */
#define VENDOR_PIONEER		3	/* Use Pioneer audio commands */
#define VENDOR_HITACHI		4	/* Use Hitachi audio commands */


/* Data direction code */
#define READ_OP			0	/* SCSI data direction */
#define WRITE_OP		1	/* SCSI data direction */


/* Play audio format codes */
#define ADDR_BLK		0x01	/* block address specified */
#define ADDR_MSF		0x02	/* MSF address specified */
#define ADDR_TRKIDX		0x04	/* track/index numbers specified */
#define ADDR_OPTEND		0x80	/* End address can be ignored */


/* Misc constants */
#define MAX_SRCH_BLKS		225	/* max search play blks per sample */
#define MAX_SCSIVOL		0xff	/* SCSI Mode Sense/Mode Select audio
					 * volume maximum value.
					 */
#define MAX_RECOVERR		20	/* Max number of err recovery tries */
#define ERR_SKIPBLKS		10	/* Number of frame to skip on error */
#define ERR_CLRTHRESH		1500	/* If there hasn't been any errors
					 * for this many blocks of audio
					 * playback, then the previous errors
					 * count is cleared.
					 */


/* SCSI command opcodes */

/* 6-byte commands */
#define OP_S_TEST		0x00	/* test unit ready */
#define OP_S_REZERO		0x01	/* rezero */
#define OP_S_RSENSE		0x03	/* request sense */
#define OP_S_SEEK		0x0b	/* seek */
#define OP_S_INQUIR		0x12	/* inquiry */
#define OP_S_MSELECT		0x15	/* mode select */
#define OP_S_MSENSE		0x1a	/* mode sense */
#define OP_S_START		0x1b	/* start/stop unit */
#define OP_S_PREVENT		0x1e	/* prevent/allow medium removal */

/* 10-byte commands */
#define OP_M_RDCAP		0x25	/* read capacity */
#define OP_M_RDSUBQ		0x42	/* read subchannel */
#define OP_M_RDTOC		0x43	/* read TOC */
#define OP_M_RDHDR		0x44	/* read header */
#define OP_M_PLAY		0x45	/* play audio */
#define OP_M_PLAYMSF		0x47	/* play audio MSF */
#define OP_M_PLAYTI		0x48	/* play audio track/index */
#define OP_M_PLAYTR		0x49	/* play audio track relative */
#define OP_M_PAUSE		0x4b	/* pause/resume */

/* 12-byte commands */
#define OP_L_PLAY		0xa5	/* play audio */
#define OP_L_PLAYTR		0xa9	/* play audio track relative */


/* Data buffer lengths */
#define SZ_RDSUBQ		48	/* max read sub-channel Q data size */
#define SZ_RDTOC		804	/* max read TOC data size */
#define SZ_TOCHDR		4	/* TOC header size */
#define SZ_TOCENT		8	/* TOC per-track entry size */
#define SZ_MSENSE		60	/* max mode sense/mode sel data size */
#define SZ_RSENSE		18	/* max request sense data size */

/* Data buffer address */
#define AD_RSENSE(x)		((byte_t *) (x) + 1)
					/* start of real request sense data */


/* Read Sub-channel audio status */
#define AUDIO_NOTVALID		0x00	/* audio status not valid */
#define AUDIO_PLAYING		0x11	/* audio play in progress */
#define AUDIO_PAUSED		0x12	/* audio play paused */
#define AUDIO_COMPLETED		0x13	/* audio play successfully completed */
#define AUDIO_FAILED		0x14	/* audio played stopped due to error */
#define AUDIO_NOSTATUS		0x15	/* no audio status */


/* Mode sense/mode select page codes */
#define PG_ERRECOV		0x01	/* error recovery parameters page */
#define PG_DISCONN		0x02	/* disconn/conn parameters page */
#define PG_CDROMCTL		0x0d	/* cd-rom control parameters page */
#define PG_AUDIOCTL		0x0e	/* audio control parameters page */
#define PG_ALL			0x3f	/* 0x01, 0x02, 0x0d and 0x0e */


/* Read sub-channel format codes */
#define SUB_ALL			0x00	/* sub-Q channel data */
#define SUB_CURPOS		0x01	/* current CD-ROM position */
#define SUB_CATNO		0x02	/* media catalog num (UPC/bar code) */
#define SUB_ISRC		0x03	/* track ISRC code */


/* Inquiry data misc definitions */
#define DEV_ROM			0x05	/* ROM peripheral device type */
#define DEV_CONNECTED		0x00	/* peripheral qualifier */


/* The inquiry data structure */
typedef struct inquiry_data {
#if BYTE_ORDER == LITTLE_ENDIAN
	unsigned int	type:5;		/* peripheral device type */
	unsigned int	pqual:3;	/* peripheral qualifier */
	unsigned int	qualif:7;	/* device type qualifier */
	unsigned int	rmb:1;		/* removable media */
#else	/* BYTE_ORDER == BIG_ENDIAN */
	unsigned int	pqual:3;	/* peripheral qualifier */
	unsigned int	type:5;		/* peripheral device type */
	unsigned int	rmb:1;		/* removable media */
	unsigned int	qualif:7;	/* device type qualifier */
#endif	/* BYTE_ORDER */

	unsigned int	ver:8;		/* SCSI version */
	unsigned int	res1:8;		/* reserved */

	byte_t		len;		/* length of additional data */
	byte_t		res2[3];	/* reserved */
	byte_t		vendor[8];	/* vendor ID */
	byte_t		prod[16];	/* product ID */
	byte_t		revnum[4];	/* revision number */
} inquiry_data_t;


/* The Mode Sense/Mode Select data structures */

/* Block descriptor data structure */
typedef struct blk_desc {
	unsigned int	dens_code:8;	/* density code */
	unsigned int	num_blks:24;	/* number of blocks */

	unsigned int	res:8;		/* reserved */
	unsigned int	blk_len:24;	/* block length */
} blk_desc_t;

/* Audio-parameters page data structure */
typedef struct audio_pg {
#if BYTE_ORDER == LITTLE_ENDIAN
	unsigned int	pg_code:6;	/* page code */
	unsigned int	res:2;		/* reserved */
	unsigned int	pg_len:8;	/* page length */
	unsigned int	res1:1;		/* reserved */
	unsigned int	sotc:1;		/* SOTC */
	unsigned int	immed:1;	/* immediate */
	unsigned int	res2:5;		/* reserved */
	unsigned int	res3:8;		/* reserved */

	unsigned int	res4:16;	/* reserved */
	unsigned int	audio_bps:16;	/* logical blocks per second */

	unsigned int	p0_ch_ctrl:4;	/* port 0 channel control */
	unsigned int	res5:4;		/* reserved */
	unsigned int	p0_vol:8;	/* port 0 volume */
	unsigned int	p1_ch_ctrl:4;	/* port 1 channel control */
	unsigned int	res6:4;		/* reserved */
	unsigned int	p1_vol:8;	/* port 1 volume */

	unsigned int	p2_ch_ctrl:4;	/* port 2 channel control */
	unsigned int	res7:4;		/* reserved */
	unsigned int	p2_vol:8;	/* port 2 volume */
	unsigned int	p3_ch_ctrl:4;	/* port 3 channel control */
	unsigned int	res8:4;		/* reserved */
	unsigned int	p3_vol:8;	/* port 3 volume */
#else	/* BYTE_ORDER == BIG_ENDIAN */
	unsigned int	res:2;		/* reserved */
	unsigned int	pg_code:6;	/* page code */
	unsigned int	pg_len:8;	/* page length */
	unsigned int	res2:5;		/* reserved */
	unsigned int	immed:1;	/* immediate */
	unsigned int	sotc:1;		/* SOTC */
	unsigned int	res1:1;		/* reserved */
	unsigned int	res3:8;		/* reserved */

	unsigned int	res4:16;	/* reserved */
	unsigned int	audio_bps:16;	/* logical blocks per second */

	unsigned int	res5:4;		/* reserved */
	unsigned int	p0_ch_ctrl:4;	/* port 0 channel control */
	unsigned int	p0_vol:8;	/* port 0 volume */
	unsigned int	res6:4;		/* reserved */
	unsigned int	p1_ch_ctrl:4;	/* port 1 channel control */
	unsigned int	p1_vol:8;	/* port 1 volume */

	unsigned int	res7:4;		/* reserved */
	unsigned int	p2_ch_ctrl:4;	/* port 2 channel control */
	unsigned int	p2_vol:8;	/* port 2 volume */
	unsigned int	res8:4;		/* reserved */
	unsigned int	p3_ch_ctrl:4;	/* port 3 channel control */
	unsigned int	p3_vol:8;	/* port 3 volume */
#endif	/* BYTE_ORDER */
} audio_pg_t;

/* Mode Sense/Mode Select data structure */
typedef struct mode_sense_data {
	/* mode header */
	unsigned int	data_len:8;	/* data length */
	unsigned int	medium:8;	/* medium */
#if BYTE_ORDER == LITTLE_ENDIAN
	unsigned int	speed:4;	/* speed */
	unsigned int	buffered:3;	/* buffered */
	unsigned int	wprot:1;	/* write protected */
#else	/* BYTE_ORDER == BIG_ENDIAN */
	unsigned int	wprot:1;	/* write protected */
	unsigned int	buffered:3;	/* buffered */
	unsigned int	speed:4;	/* speed */
#endif	/* BYTE_ORDER */
	unsigned int	bdescr_len:8;	/* block descriptor length */

	byte_t		data[24];	/* block desc/page desc data */
} mode_sense_data_t;


/* Request Sense data structure */
typedef struct req_sense_data {
	unsigned int	pad:8;		/* pad byte for alignment */
#if BYTE_ORDER == LITTLE_ENDIAN
	unsigned int	errcode:7;	/* error code */
	unsigned int	valid:1;	/* valid bit */
	unsigned int	segno:8;	/* segment number */
	unsigned int	key:4;		/* sense key */
	unsigned int	res:1;		/* reserved */
	unsigned int	ili:1;		/* ILI */
	unsigned int	eom:1;		/* end-of-medium */
	unsigned int	fm:1;		/* filemark */
#else	/* BYTE_ORDER == BIG_ENDIAN */
	unsigned int	valid:1;	/* valid bit */
	unsigned int	errcode:7;	/* error code */
	unsigned int	segno:8;	/* segment number */
	unsigned int	fm:1;		/* filemark */
	unsigned int	eom:1;		/* end-of-medium */
	unsigned int	ili:1;		/* ILI */
	unsigned int	res:1;		/* reserved */
	unsigned int	key:4;		/* sense key */
#endif	/* BYTE_ORDER */

	byte_t		info[4];	/* information */

	byte_t	 	addl_len;	/* additional sense length */
	byte_t		cmd_spec[4];	/* command specific information */

	byte_t		code;		/* additional sense code */
	byte_t		qual;		/* additional sense code qualifier */

	byte_t		fruc;		/* field replaceable unit code */
	byte_t		key_spec[3];	/* sense-key specific */
} req_sense_data_t;


/* CD position MSF structure */
typedef struct msf {
	byte_t		res;		/* reserved */
	byte_t		min;		/* minutes */
	byte_t		sec;		/* seconds */
	byte_t		frame;		/* frame */
} msf_t;


/* Combined MSF and logical address union */
typedef union lmsf {
	msf_t		msf;		/* MSF address */
	word32_t	logical;	/* logical address */
} lmsf_t;


/* Read subchannel Q data header */
typedef struct subq_hdr {
	byte_t		reserved;	/* reserved */
	byte_t		audio_status;	/* audio status */
	word16_t	subch_len;	/* subchannel data length */
} subq_hdr_t;


/* Subchannel Q data - format 01 (CD-ROM Current Position) */
typedef struct subq_01 {
	unsigned int	fmt_code:8;	/* format code */
#if BYTE_ORDER == LITTLE_ENDIAN
	unsigned int	preemph:1;	/* preemphasis */
	unsigned int	copyallow:1;	/* digital copy allow */
	unsigned int	trktype:1;	/* 0=audio 1=data */
	unsigned int	audioch:1;	/* 0=2ch 1=4ch */
	unsigned int	adr:4;		/* ADR */
#else	/* BYTE_ORDER == BIG_ENDIAN */
	unsigned int	adr:4;		/* ADR */
	unsigned int	audioch:1;	/* 0=2ch 1=4ch */
	unsigned int	trktype:1;	/* 0=audio 1=data */
	unsigned int	copyallow:1;	/* digital copy allow */
	unsigned int	preemph:1;	/* preemphasis */
#endif	/* BYTE_ORDER */
	unsigned int	trkno:8;	/* track number */
	unsigned int	idxno:8;	/* index number */

	lmsf_t		abs_addr;	/* absolute address */
	lmsf_t		rel_addr;	/* track-relative address */
} subq_01_t;


/* Read TOC command data header */
typedef struct toc_hdr {
	word16_t	data_len;	/* TOC data length */
	byte_t		first_trk;	/* first track number */
	byte_t		last_trk;	/* last track number */
} toc_hdr_t;


/* Read TOC command track descriptor */
typedef struct toc_trk_descr {
	unsigned int	res1:8;		/* reserved */
#if BYTE_ORDER == LITTLE_ENDIAN
	unsigned int	preemph:1;	/* preemphasis */
	unsigned int	copyallow:1;	/* digital copy allow */
	unsigned int	trktype:1;	/* 0=audio 1=data */
	unsigned int	audioch:1;	/* 0=2ch 1=4ch */
	unsigned int	adr:4;		/* ADR */
#else	/* BYTE_ORDER == BIG_ENDIAN */
	unsigned int	adr:4;		/* ADR */
	unsigned int	audioch:1;	/* 0=2ch 1=4ch */
	unsigned int	trktype:1;	/* 0=audio 1=data */
	unsigned int	copyallow:1;	/* digital copy allow */
	unsigned int	preemph:1;	/* preemphasis */
#endif	/* BYTE_ORDER */
	unsigned int	trkno:8;	/* track number */
	unsigned int	res2:8;		/* reserved */

	lmsf_t		abs_addr;	/* absolute address */
} toc_trk_descr_t;


/***** Additional include files *****/

/* OS interface library headers */
#include "lib_hpux.h"			/* HP-UX support header */
#include "lib_odt.h"			/* SCO ODT support header */
#include "lib_sun.h"			/* SunOS support header */
#include "lib_svr4.h"			/* SVR4 support header */
#include "lib_demo.h"			/* Demo-only support header -
					 * This must be included last.
					 */

/* Vendor-unique library headers */
#ifdef HITACHI
#include "lib_hita.h"			/* Hitachi vendor-unique header */
#endif
#ifdef NEC
#include "lib_nec.h"			/* NEC vendor-unique header */
#endif
#ifdef PIONEER
#include "lib_pion.h"			/* Pioneer vendor-unique header */
#endif
#ifdef TOSHIBA
#include "lib_tosh.h"			/* Toshiba vendor-unique header */
#endif


#endif	/* __LIB_SCSIPT_H__ */

